﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using Datastream.DswsApi;

namespace SampleCode.DswsApi.Net40
{
    /// <summary>
    /// Contains sample code related to Asynchronous access
    /// </summary>
    internal class AsyncAccess : IAsyncAccess
    {
        #region Members

        public event Action<Exception, string> AsyncOperationCompleted;

        static AsyncAccess()
        {

        } //end static AsyncAccess


        public void GetDataAsync()
        {
            // Gets data asynchronously
            // Change the instrument, datatypes and date appropriately

            // Form the request
            var request = new DSDataRequest()
            {
                Instrument = new DSInstrument("VOD", "IBM", "MKS"),
                DataTypes = new DSDataTypes("PL", "PH"),
                Date = new DSTimeSeriesDate(DSDateType.Relative(-30), null, DSDateFrequency.Weekly),
            };

            // Execute the request asynchronously
            // As soon as the execution is over, the continuewith lambda will be executed
            var task = DSClient.DataServiceAsync.GetDataAsync(request);
            task.ContinueWith(t =>
            {
                if (t.Exception != null)
                {
                    if (this.AsyncOperationCompleted != null)
                        this.AsyncOperationCompleted(t.Exception, null);
                }
                else
                {
                    // Return response as string (for display purpose)
                    string response = GetResponseString(t.Result);
                    if (this.AsyncOperationCompleted != null)
                        this.AsyncOperationCompleted(null, response);
                }
            });


        } //end GetDataAsync

        public void GetDataBundleAsync()
        {
            // Gets data bundle asynchronously
            // Change the instrument, datatypes and date appropriately

            // Form the requests
            var requests = new[]
            {
                new DSDataRequest()
                {
                    Instrument = new DSInstrument("VOD", "IBM", "MKS"),
                    DataTypes = new DSDataTypes("MV", "APC"),
                    Date = new DSTimeSeriesDate(DSDateType.Relative(-30), null, DSDateFrequency.Weekly),
                },
                new DSDataRequest()
                {
                    Instrument = new DSInstrument("LFTSE100", DSInstrumentProperties.IsList),
                    DataTypes = new DSDataTypes("MNEM", "NAME"),
                    Date = new DSSnapshotDate(null),
                },
            };

            // Execute the request asynchronously
            // As soon as the execution is over, the continuewith lambda will be executed
            var task = DSClient.DataServiceAsync.GetDataBundleAsync(requests);
            task.ContinueWith(t =>
            {
                if (t.Exception != null)
                {
                    if (this.AsyncOperationCompleted != null)
                        this.AsyncOperationCompleted(t.Exception, null);
                }
                else
                {
                    // Return response as string (for display purpose)
                    string response = GetResponsesString(t.Result);
                    if (this.AsyncOperationCompleted != null)
                        this.AsyncOperationCompleted(null, response);
                }
            });


        } //end GetDataBundleAsync

        public void GetDataAsync_Arbitrary(DSDataRequest request)
        {
            // Execute the request asynchronously
            // As soon as the execution is over, the continuewith lambda will be executed
            var task = DSClient.DataServiceAsync.GetDataAsync(request);
            task.ContinueWith(t =>
            {
                if (t.Exception != null)
                {
                    if (this.AsyncOperationCompleted != null)
                        this.AsyncOperationCompleted(t.Exception, null);
                }
                else
                {
                    // Return response as string (for display purpose)
                    string response = GetResponseString(t.Result);
                    if (this.AsyncOperationCompleted != null)
                        this.AsyncOperationCompleted(null, response);
                }
            });



        } //end GetDataAsync_Arbitrary

        public void GetDataBundleAsync_Arbitrary(DSDataRequest[] requests)
        {
            // Execute the request asynchronously
            // As soon as the execution is over, the continuewith lambda will be executed
            var task = DSClient.DataServiceAsync.GetDataBundleAsync(requests);
            task.ContinueWith(t =>
            {
                if (t.Exception != null)
                {
                    if (this.AsyncOperationCompleted != null)
                        this.AsyncOperationCompleted(t.Exception, null);
                }
                else
                {
                    // Return response as string (for display purpose)
                    string response = GetResponsesString(t.Result);
                    if (this.AsyncOperationCompleted != null)
                        this.AsyncOperationCompleted(null, response);
                }
            });



        } //end GetDataBundleAsync_Arbitrary

      
        #endregion

        #region Misc

        private static string GetResponsesString(IEnumerable<DSDataResponse> responses)
        {
            var sb = new StringBuilder();
            foreach (var resp in responses)
            {
                sb.Append(GetResponseString(resp));
                sb.AppendLine("--------------");
            }
            return sb.ToString();

        } //end GetResponsesString

        private static string GetResponseString(DSDataResponse response)
        {
            var sb = new StringBuilder();
            foreach (var dt in response.DataTypeValues)
            {
                sb.AppendFormat("DataType:{0}", dt.DataType);
                sb.AppendLine();
                foreach (var sym in dt.SymbolValues)
                {
                    sb.AppendFormat("   {0},{1},{2},{3}", sym.Symbol, sym.Currency, sym.Type, sym.Value is Array ? ((Array)sym.Value).GetLength(0).ToString() + sym.Value.ToString() : sym.Value.ToString());
                    sb.AppendLine();
                }
            }

            // Datatype names
            if (response.DataTypeNames != null)
            {
                sb.Append("DataTypeNames:");
                sb.AppendLine();
                sb.Append(string.Join(",", response.DataTypeNames.Select(x => string.Format("{0}=>{1}", x.Key, x.Value)).ToArray()));
                sb.AppendLine();
            }

            // Symbol names
            if (response.SymbolNames != null)
            {
                sb.Append("SymbolNames:");
                sb.AppendLine();
                sb.Append(string.Join(",", response.SymbolNames.Select(x => string.Format("{0}=>{1}", x.Key, x.Value)).ToArray()));
                sb.AppendLine();
            }

            // Dates
            if (response.Dates != null)
            {
                sb.Append("Dates:");
                sb.AppendLine();
                sb.Append(response.Dates.Length);
                sb.AppendLine();
            }

            return sb.ToString();

        } //end GetResponseString

        #endregion

    }
}
