﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using Datastream.DswsApi;

namespace SampleCode.DswsApi.Net40
{
    /// <summary>
    /// Contains sample code related to GetData
    /// </summary>
    internal class GetData
    {
        #region Snapshot

        public string GetData_SnapshotSingle()
        {
            // Gets snapshot data for a single instrument
            // Change the instrument, datatypes and date appropriately

            // Form the request
            var request = new DSDataRequest()
            {
                Instrument = new DSInstrument("VOD"),
                DataTypes = new DSDataTypes("PL", "PH"),
                Date = new DSSnapshotDate(DSDateType.Relative(-30, DSDateFrequency.Daily)),
            };

            // Execute the request
            var response = DSClient.DataService.GetData(request);

            // Access the response and return as string (for display purpose)
            var respValue1 = response["PL"]["VOD"];
            var respValue2 = response["PH"]["VOD"];
            return string.Format("{0}{1}, {2}{3}", 
                respValue1.Currency, respValue1.Value, 
                respValue2.Currency, respValue2.Value);

        } //end GetData_SnapshotSingle


        public string GetData_SnapshotMulti()
        {
            // Gets snapshot data for multiple instruments
            // Change the instrument, datatypes and date appropriately

            // Form the request
            var request = new DSDataRequest()
            {
                Instrument = new DSInstrument("IBM", "MKS", "@MSFT"),
                DataTypes = new DSDataTypes("MV", "APC"),
                Date = new DSSnapshotDate(DSDateType.Absolute(DateTime.Now.AddDays(-10))),
                Properties = new DSDataRequestProperties(DSDataRequestPropertyTypes.ReturnDataTypeExpandedName),
            };

            // Execute the request
            var response = DSClient.DataService.GetData(request);

            // Access the response and return as string (for display purpose)
            var dtRespValue1 = response["MV"];
            var dtRespValue2 = response["APC"];
            return string.Format("{0},{1},{2},{3},{4},{5}", 
                dtRespValue1["IBM"].Value, dtRespValue1["MKS"].Value, dtRespValue1["@MSFT"].Value, 
                dtRespValue2["IBM"].Value, dtRespValue2["MKS"].Value, dtRespValue2["@MSFT"].Value);


        } //end GetData_SnapshotMulti


        public string GetData_SnapshotExpression()
        {
            // Gets snapshot data for an expression
            // Change the instrument, datatypes and date appropriately

            // Form the request
            var request = new DSDataRequest()
            {
                Instrument = new DSInstrument("VOD(PL) + BARC(PH) / 3", DSInstrumentProperties.IsExpression),
                Date = new DSSnapshotDate(DSDateType.Literal(DSDateLiterals.MonthToEnd)),
            };

            // Execute the request
            var response = DSClient.DataService.GetData(request);

            // Return response as string (for display purpose)
            return GetResponseString(response);

        } //end GetData_SnapshotExpression


        public string GetData_SnapshotList()
        {
            // Gets snapshot data for list instrument
            // Change the instrument, datatypes and date appropriately

            // Form the request
            var request = new DSDataRequest()
            {
                Instrument = new DSInstrument("LFTSE100", DSInstrumentProperties.IsList),
                DataTypes = new DSDataTypes("MNEM", "NAME"),
                Date = new DSSnapshotDate(null),
            };

            // Execute the request
            var response = DSClient.DataService.GetData(request);

            // Return response as string (for display purpose)
            return GetResponseString(response);

        } //end GetData_SnapshotList


        #endregion

        #region TimeSeries

        public string GetData_TimeSeriesSingle()
        {
            // Gets timeseries data for a single instrument
            // Change the instrument, datatypes and date appropriately

            // Form the request
            var request = new DSDataRequest()
            {
                Instrument = new DSInstrument("VOD"),
                DataTypes = new DSDataTypes("PL", "PH"),
                Date = new DSTimeSeriesDate(DSDateType.Relative(-30, DSDateFrequency.Daily), DSDateType.Relative(-0, DSDateFrequency.Daily), DSDateFrequency.Daily),
            };

            // Execute the request
            var response = DSClient.DataService.GetData(request);

            // Access the response and return as string (for display purpose)
            var respValue1 = response["PL"]["VOD"];
            var respValue2 = response["PH"]["VOD"];
            return string.Format("{0} {1}, {2} {3}",
                respValue1.Currency, respValue1.Value is Array ? ((Array)respValue1.Value).Length + " items" : respValue1.Value,
                respValue2.Currency, respValue2.Value is Array ? ((Array)respValue2.Value).Length + " items" : respValue2.Value);

        } //end GetData_TimeSeriesSingle


        public string GetData_TimeSeriesMulti()
        {
            // Gets timeseries data for multiple instruments
            // Change the instrument, datatypes and date appropriately

            // Form the request
            var request = new DSDataRequest()
            {
                Instrument = new DSInstrument("VOD", "IBM", "MKS"),
                DataTypes = new DSDataTypes("PL", "PH"),
                Date = new DSTimeSeriesDate(DSDateType.Absolute(DateTime.Now.AddMonths(-20)), DSDateType.Absolute(DateTime.Now), DSDateFrequency.Weekly),
                Properties = new DSDataRequestProperties(DSDataRequestPropertyTypes.ReturnDataTypeExpandedName | DSDataRequestPropertyTypes.ReturnInstrumentExpandedName),
            };

            // Execute the request
            var response = DSClient.DataService.GetData(request);

            // Return response as string (for display purpose)
            return GetResponseString(response);

        } //end GetData_TimeSeriesMulti


        public string GetData_TimeSeriesExpression()
        {
            // Gets timeseries data for an expression
            // Change the instrument, datatypes and date appropriately

            // Form the request
            var request = new DSDataRequest()
            {
                Instrument = new DSInstrument("VOD(PL) + BARC(PH) / 3", DSInstrumentProperties.IsExpression),
                Date = new DSTimeSeriesDate(DSDateType.Literal(DSDateLiterals.StartDateOfYear), DSDateType.Literal(DSDateLiterals.Today), DSDateFrequency.Monthly),
            };

            // Execute the request
            var response = DSClient.DataService.GetData(request);

            // Return response as string (for display purpose)
            return GetResponseString(response);

        } //end GetData_TimeSeriesExpression

        public string GetData_Arbitrary(DSDataRequest request)
        {
            // Execute the request
            var response = DSClient.DataService.GetData(request);

            // Return response as string (for display purpose)
            return GetResponseString(response);

        } //end GetData_Arbitrary


        #endregion

        #region Misc

        private string GetResponseString(DSDataResponse response)
        {
            var sb = new StringBuilder();
            foreach (var dt in response.DataTypeValues)
            {
                sb.AppendFormat("DataType:{0}", dt.DataType);
                sb.AppendLine();
                foreach (var sym in dt.SymbolValues)
                {
                    sb.AppendFormat("   {0},{1},{2},{3}", sym.Symbol, sym.Currency, sym.Type, sym.Value is Array ? ((Array)sym.Value).GetLength(0).ToString() + sym.Value.ToString() : sym.Value.ToString());
                    sb.AppendLine();
                }
            }

            // Datatype names
            if (response.DataTypeNames != null)
            {
                sb.Append("DataTypeNames:");
                sb.AppendLine();
                sb.Append(string.Join(",", response.DataTypeNames.Select(x => string.Format("{0}=>{1}", x.Key, x.Value)).ToArray()));
                sb.AppendLine();
            }

            // Symbol names
            if (response.SymbolNames != null)
            {
                sb.Append("SymbolNames:");
                sb.AppendLine();
                sb.Append(string.Join(",", response.SymbolNames.Select(x => string.Format("{0}=>{1}", x.Key, x.Value)).ToArray()));
                sb.AppendLine();
            }

            // Dates
            if (response.Dates != null)
            {
                sb.Append("Dates:");
                sb.AppendLine();
                sb.Append(response.Dates.Length);
                sb.AppendLine();
            }

            return sb.ToString();

        } //end GetResponseString

        #endregion

    }
}
