﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using Datastream.DswsApi;

namespace SampleCode.DswsApi.Net40
{
    /// <summary>
    /// Contains sample code related to GetDataBundle
    /// </summary>
    internal class GetDataBundle
    {
        #region Snapshot

        public string GetDataBundle_Snapshot()
        {
            // Gets snapshot data for multiple requests
            // Change the instrument, datatypes and date appropriately

            // Form the request
            var requests = new[]
            {
                new DSDataRequest()
                {
                    Instrument = new DSInstrument("VOD"),
                    DataTypes = new DSDataTypes("PL", "PH"),
                    Date = new DSSnapshotDate(DSDateType.Relative(-30, DSDateFrequency.Daily)),
                },
                new DSDataRequest()
                {
                    Instrument = new DSInstrument("IBM", "MKS"),
                    DataTypes = new DSDataTypes("MV", "APC"),
                    Date = new DSSnapshotDate(DSDateType.Absolute(DateTime.Now.AddDays(-10))),
                    Properties = new DSDataRequestProperties(DSDataRequestPropertyTypes.ReturnDataTypeExpandedName),
                },
                new DSDataRequest()
                {
                    Instrument = new DSInstrument("VOD(PL) + BARC(PH) / 3", DSInstrumentProperties.IsExpression),
                    Date = new DSSnapshotDate(DSDateType.Literal(DSDateLiterals.MonthToEnd)),
                },
                new DSDataRequest()
                {
                    Instrument = new DSInstrument("LFTSE100", DSInstrumentProperties.IsList),
                    DataTypes = new DSDataTypes("MNEM", "NAME"),
                    Date = new DSSnapshotDate(null),
                }
            };

            // Execute the request
            var responses = DSClient.DataService.GetDataBundle(requests);

            // Return responses as string (for display purpose)
            return GetResponsesString(responses);

        } //end GetDataBundle_Snapshot


        public string GetDataBundle_TimeSeries()
        {
            // Gets timeseries data for multiple requests
            // Change the instrument, datatypes and date appropriately

            // Form the request
            var requests = new[]
            {
                new DSDataRequest()
                {
                    Instrument = new DSInstrument("VOD"),
                    DataTypes = new DSDataTypes("PL", "PH"),
                    Date = new DSTimeSeriesDate(DSDateType.Relative(-30, DSDateFrequency.Daily), DSDateType.Relative(-0, DSDateFrequency.Daily), DSDateFrequency.Daily),
                },
                new DSDataRequest()
                {
                    Instrument = new DSInstrument("VOD", "IBM", "MKS"),
                    DataTypes = new DSDataTypes("MV", "APC"),
                    Date = new DSTimeSeriesDate(DSDateType.Absolute(DateTime.Now.AddMonths(-20)), DSDateType.Absolute(DateTime.Now), DSDateFrequency.Weekly),
                    Properties = new DSDataRequestProperties(DSDataRequestPropertyTypes.ReturnDataTypeExpandedName | DSDataRequestPropertyTypes.ReturnInstrumentExpandedName),
                },
                new DSDataRequest()
                {
                    Instrument = new DSInstrument("VOD(PL) + BARC(PH) / 3", DSInstrumentProperties.IsExpression),
                    Date = new DSTimeSeriesDate(DSDateType.Literal(DSDateLiterals.StartDateOfYear), DSDateType.Literal(DSDateLiterals.Today), DSDateFrequency.Monthly),
                },
            };

            // Execute the request
            var responses = DSClient.DataService.GetDataBundle(requests);

            // Return responses as string (for display purpose)
            return GetResponsesString(responses);

        } //end GetDataBundle_TimeSeries

        public string GetDataBundle_Mixed()
        {
            // Gets snapshot/timeseries data for multiple requests
            // Change the instrument, datatypes and date appropriately

            // Form the request
            var requests = new[]
            {
                new DSDataRequest()
                {
                    Instrument = new DSInstrument("VOD"),
                    DataTypes = new DSDataTypes("PL", "PH"),
                    Date = new DSSnapshotDate(DSDateType.Relative(-30, DSDateFrequency.Daily)),
                },
                new DSDataRequest()
                {
                    Instrument = new DSInstrument("VOD", "IBM", "MKS"),
                    DataTypes = new DSDataTypes("MV", "APC"),
                    Date = new DSTimeSeriesDate(DSDateType.Absolute(DateTime.Now.AddMonths(-20)), DSDateType.Absolute(DateTime.Now), DSDateFrequency.Weekly),
                    Properties = new DSDataRequestProperties(DSDataRequestPropertyTypes.ReturnDataTypeExpandedName | DSDataRequestPropertyTypes.ReturnInstrumentExpandedName),
                },
                new DSDataRequest()
                {
                    Instrument = new DSInstrument("LFTSE100", DSInstrumentProperties.IsList),
                    DataTypes = new DSDataTypes("MNEM", "NAME"),
                    Date = new DSSnapshotDate(null),
                },
                new DSDataRequest()
                {
                    Instrument = new DSInstrument("VOD(PL) + BARC(PH) / 3", DSInstrumentProperties.IsExpression),
                    Date = new DSTimeSeriesDate(DSDateType.Literal(DSDateLiterals.StartDateOfYear), DSDateType.Literal(DSDateLiterals.Today), DSDateFrequency.Monthly),
                },
                
            };

            // Execute the request
            var responses = DSClient.DataService.GetDataBundle(requests);

            // Return responses as string (for display purpose)
            return GetResponsesString(responses);

        } //end GetDataBundle_Mixed


        public string GetDataBundle_Arbitrary(DSDataRequest[] requests)
        {
            // Execute the requests
            var responses = DSClient.DataService.GetDataBundle(requests);

            // Return responses as string (for display purpose)
            return GetResponsesString(responses);

        } //end GetDataBundle_Arbitrary


       

        #endregion

        #region Misc

        private string GetResponsesString(IEnumerable<DSDataResponse> responses)
        {
            var sb = new StringBuilder();
            foreach (var resp in responses)
            {
                sb.Append(GetResponseString(resp));
                sb.AppendLine("--------------");
            }
            return sb.ToString();

        } //end GetResponsesString

        private string GetResponseString(DSDataResponse response)
        {
            var sb = new StringBuilder();
            foreach (var dt in response.DataTypeValues)
            {
                sb.AppendFormat("DataType:{0}", dt.DataType);
                sb.AppendLine();
                foreach (var sym in dt.SymbolValues)
                {
                    sb.AppendFormat("   {0},{1},{2},{3}", sym.Symbol, sym.Currency, sym.Type, sym.Value is Array ? ((Array)sym.Value).GetLength(0).ToString() + sym.Value.ToString() : sym.Value.ToString());
                    sb.AppendLine();
                }
            }

            // Datatype names
            if (response.DataTypeNames != null)
            {
                sb.Append("DataTypeNames:");
                sb.AppendLine();
                sb.Append(string.Join(",", response.DataTypeNames.Select(x => string.Format("{0}=>{1}", x.Key, x.Value)).ToArray()));
                sb.AppendLine();
            }

            // Symbol names
            if (response.SymbolNames != null)
            {
                sb.Append("SymbolNames:");
                sb.AppendLine();
                sb.Append(string.Join(",", response.SymbolNames.Select(x => string.Format("{0}=>{1}", x.Key, x.Value)).ToArray()));
                sb.AppendLine();
            }

            // Dates
            if (response.Dates != null)
            {
                sb.Append("Dates:");
                sb.AppendLine();
                sb.Append(response.Dates.Length);
                sb.AppendLine();
            }

            return sb.ToString();

        } //end GetResponseString

        #endregion

    }
}
